@@ -45,12 +45,17 @@ mappingP oid encode decode v =
     unsafePerformIO $ do
       c <- connect
       initConnection c
-      Just result <-
+      result <-
         let param = (,,) <$> pure (PQ.Oid $ fromIntegral oid) <*> encode v <*> pure PQ.Binary
             in PQ.execParams c "SELECT $1" [param] PQ.Binary
-      binaryResult <- PQ.getvalue result 0 0
-      PQ.finish c
-      return $ decode binaryResult
+      case result of
+        Just result -> do
+          binaryResult <- PQ.getvalue result 0 0
+          PQ.finish c
+          return $ decode binaryResult
+        Nothing -> do
+          m <- PQ.errorMessage c
+          fail $ maybe "Fatal PQ error" (\m -> "Fatal PQ error: " <> show m) m
 
 mappingTextP ::
   (Show a, Eq a) => 
@@ -199,6 +204,36 @@ maxInterval :: DiffTime =
 minInterval :: DiffTime = 
   negate maxInterval
 
+integerDatetimes :: Bool
+integerDatetimes =
+  unsafePerformIO $ do
+    connection <- connect
+    initConnection connection
+    integerDatetimes <- getIntegerDatetimes connection
+    PQ.finish connection
+    return integerDatetimes
+
+-- * Misc
+-------------------------
+
+timestamptzApxRep (UTCTime d d') =
+  (d, picoApxRep (unsafeCoerce d'))
+
+timestampApxRep (LocalTime d t) =
+  (d, timeApxRep t)
+
+timetzApxRep (t, tz) = 
+  (timeApxRep t, tz)
+
+timeApxRep (TimeOfDay h m s) =
+  (h, m, picoApxRep s)
+
+picoApxRep :: Pico -> Integer
+picoApxRep s =
+  let p = unsafeCoerce s :: Integer
+      in round (p % 10^6)
+
+
 -- * Tests
 -------------------------
 
@@ -255,46 +290,49 @@ test_minInterval =
           query "SELECT $1" [Just p] PQ.Binary
 
 prop_timestamp =
-  forAll microsUTCTimeGen $ 
-    mappingP (PTI.oidOf PTI.timestamp) 
-             (nonNullRenderer Encoder.timestamp)
-             (nonNullParser Decoder.timestamp)
+  forAll microsLocalTimeGen $ \x ->
+    Just (Right (timestampApxRep x)) === do
+      unsafePerformIO $ do
+        let p = (,,) (PQ.Oid $ fromIntegral $ PTI.oidOf PTI.timestamp)
+                     (Encoder.timestamp integerDatetimes x)
+                     (PQ.Binary)
+            in (fmap . fmap) (fmap timestampApxRep . Decoder.timestamp integerDatetimes)
+                             (query "SELECT $1" [Just p] PQ.Binary)
 
 test_timestampParsing1 =
-  assertEqual (Right (read "2000-01-19 10:41:06" :: UTCTime)) =<< do
-    fmap (Decoder.timestamp . fromJust) $ 
+  assertEqual (Right (read "2000-01-19 10:41:06" :: LocalTime)) =<< do
+    fmap (Decoder.timestamp integerDatetimes . fromJust) $ 
       query "SELECT '2000-01-19 10:41:06' :: timestamp" [] PQ.Binary
 
 prop_timestamptz =
-  forAll microsLocalTimeGen $ 
-    mappingP (PTI.oidOf PTI.timestamptz) 
-             (nonNullRenderer Encoder.timestamptz)
-             (nonNullParser Decoder.timestamptz)
+  forAll microsUTCTimeGen $ \x ->
+    Just (Right (timestamptzApxRep x)) === do
+      unsafePerformIO $ do
+        let p = (,,) (PQ.Oid $ fromIntegral $ PTI.oidOf PTI.timestamptz)
+                     (Encoder.timestamptz integerDatetimes x)
+                     (PQ.Binary)
+            in (fmap . fmap) (fmap timestamptzApxRep . Decoder.timestamptz integerDatetimes)
+                             (query "SELECT $1" [Just p] PQ.Binary)
 
 prop_timetz =
   forAll ((,) <$> microsTimeOfDayGen <*> timeZoneGen) $ \x ->
-    Right x === do
+    Just (Right (timetzApxRep x)) === do
       unsafePerformIO $ do
-        connection <- connect
-        initConnection connection
-        integerDatetimes <- getIntegerDatetimes connection
-        Just result <- 
-          let params = [Just (PQ.Oid $ fromIntegral $ PTI.oidOf PTI.timetz, Encoder.timetz integerDatetimes x, PQ.Binary)]
-              in PQ.execParams connection "SELECT $1" params PQ.Binary
-        encodedResult <- PQ.getvalue result 0 0
-        PQ.finish connection
-        return $ 
-          Decoder.timetz integerDatetimes (fromJust encodedResult)
+        let p = (,,) (PQ.Oid $ fromIntegral $ PTI.oidOf PTI.timetz)
+                     (Encoder.timetz integerDatetimes x)
+                     (PQ.Binary)
+            in (fmap . fmap) (fmap timetzApxRep . Decoder.timetz integerDatetimes)
+                             (query "SELECT $1" [Just p] PQ.Binary)
 
 test_timetzParsing =
-  assertEqual (Right (read "(10:41:06.002897, +0500)" :: (TimeOfDay, TimeZone))) =<< do
+  assertEqual (Right $ timetzApxRep (read "(10:41:06.002897, +0500)" :: (TimeOfDay, TimeZone))) =<< do
     connection <- connect
     initConnection connection
     integerDatetimes <- getIntegerDatetimes connection
     Just result <- PQ.execParams connection "SELECT '10:41:06.002897+05' :: timetz" [] PQ.Binary
     encodedResult <- PQ.getvalue result 0 0
     PQ.finish connection
-    return $ 
+    return $ fmap timetzApxRep $ 
       Decoder.timetz integerDatetimes (fromJust encodedResult)
 
 prop_timeFromIntegerIsomorphism =
@@ -311,7 +349,7 @@ prop_timeFromDoubleIsomorphism =
 
 prop_time =
   forAll microsTimeOfDayGen $ \x ->
-    Right x === do
+    Right (timeApxRep x) === do
       unsafePerformIO $ do
         connection <- connect
         initConnection connection
@@ -321,12 +359,12 @@ prop_time =
               in PQ.execParams connection "SELECT $1" params PQ.Binary
         encodedResult <- PQ.getvalue result 0 0
         PQ.finish connection
-        return $ 
+        return $ fmap timeApxRep $
           Decoder.time integerDatetimes (fromJust encodedResult)
 
 prop_timeParsing =
   forAll microsTimeOfDayGen $ \x ->
-    Right x === do
+    Right (timeApxRep x) === do
       unsafePerformIO $ do
         connection <- connect
         initConnection connection
@@ -336,7 +374,7 @@ prop_timeParsing =
               in PQ.execParams connection "SELECT $1" params PQ.Binary
         encodedResult <- PQ.getvalue result 0 0
         PQ.finish connection
-        return $ 
+        return $ fmap timeApxRep $
           Decoder.time integerDatetimes (fromJust encodedResult)
 
 prop_scientific (c, e) =
